@@ -3,6 +3,7 @@ import time
 import struct
 import socket
 from zope.interface import implementer
+from twisted.internet.defer import inlineCallbacks, returnValue, deferredGenerator
 from twisted.internet import fdesc
 from twisted.python.compat import _PY3
 from twisted.internet import base, defer, address, udp, tcp
@@ -17,6 +18,7 @@ from udt4 import EASYNCRCV, ECONNLOST
 from socket import AI_PASSIVE
 from errno import EWOULDBLOCK, EINTR, EMSGSIZE, ECONNREFUSED, EAGAIN
 from twisted.python.compat import lazyByteSlice
+from collections import deque
 import gc
 _sockErrReadIgnore = [EAGAIN, EINTR, EWOULDBLOCK]
 _sockErrReadRefuse = [ECONNREFUSED]
@@ -34,26 +36,6 @@ else:
         # in the result.
         return buffer(bObj, offset) + b"".join(bArray)
 
-# class Connector2(Port2):
-
-#     def doConnect(self, fd=None):
-#         if not fd:
-#             self.socket = self.createInternetSocket()
-#             self.fileno = self.socket.fileno
-#             self.socket.connect_ex(self._connectedAddr)
-#             self._connectDone()
-#             return
-
-#         self._read(fd, self._connectedAddr)
-
-#     def doRead(self, fd=None):
-#         why = Port.doRead(self, self.socket.UDTSOCKET, self._connectedAddr)
-#         #FIXME
-#         if why:
-#             if (why[0] == ECONNLOST):
-#                 self.stopListening()
-#         return why
-
 
 @implementer(interfaces.ITCPTransport, interfaces.ISystemHandle)
 class Connection(abstract.FileDescriptor, tcp._AbortingMixin):
@@ -65,8 +47,7 @@ class Connection(abstract.FileDescriptor, tcp._AbortingMixin):
     @ivar logstr: prefix used when logging events related to this connection.
     @type logstr: C{str}
     """
-    _ttl = -1
-    _inorder = True
+    messageBuffer = deque()
     def _closeSocket(self, orderly):
         try:
             self.socket.close()
@@ -96,6 +77,10 @@ class Connection(abstract.FileDescriptor, tcp._AbortingMixin):
             data = udt.recvmsg(self.socket.UDTSOCKET, self.bufferSize)
         except udt.UDTException as ue:
             if ue[0] == EASYNCRCV:
+                #restart socket
+                ein = self.reactor._POLL_IN
+                self.reactor._poller.remove_usock(self.socket.UDTSOCKET)
+                self.reactor._poller.add_usock(self.socket.UDTSOCKET, ein)
                 return
             else:
                 return main.CONNECTION_LOST
@@ -147,77 +132,76 @@ class Connection(abstract.FileDescriptor, tcp._AbortingMixin):
         if not self.connected or self._writeDisconnected:
             return
         if data:
-            # self._tempDataBuffer.append(data)
-            # self._tempDataLen += len(data)
-            # self._ttl = ttl
-            # self._inorder = inorder
-            # self._maybePauseProducer()
-            self.startWriting()
-            l = self.writeSomeData(data, ttl, inorder)
+            self.messageBuffer.append([data, ttl, inorder])
+            #self._maybePauseProducer()
+            #self.startWriting()
+            #Really start writable event
+            try:
+                self.reactor._poller.add_usock(self.socket.UDTSOCKET,
+                                               self.reactor._POLL_OUT)
+            except udt.UDTException as ue:
+                log.msg("Error while writing:{0}".format(ue))
+                self.abortConnection()
+
+
+    _messagesAtOnce = 1024
+
+    def splitAndSend(self):
+        """
+        Split and send messages to the other side.
+        """
+        for i in xrange(self._messagesAtOnce):
+            try:
+                message, ttl, inorder = self.messageBuffer.popleft()
+            except IndexError:
+                break
+            l = self.writeSomeData(message, ttl, inorder)
+            if isinstance(l, Exception) or l < 0:
+                return l
+            #FIXME Producers are probably NOT working
+            # If there is nothing left to send,
             if l == main.CONNECTION_LOST:
                 self.loseConnection()
-            
-
-    def doWrite(self):
-        """
-        Called when data can be written.
-
-        @return: C{None} on success, an exception or a negative integer on
-            failure.
-
-        @see: L{twisted.internet.interfaces.IWriteDescriptor.doWrite}.
-        """
-        
-        if len(self.dataBuffer) - self.offset < self.SEND_LIMIT:
-            # If there is currently less than SEND_LIMIT bytes left to send
-            # in the string, extend it with the array data.
-            self.dataBuffer = _concatenate(
-                self.dataBuffer, self.offset, self._tempDataBuffer)
-            self.offset = 0
-            self._tempDataBuffer = []
-            self._tempDataLen = 0
-
-        # Send as much data as you can.
-        if self.dataBuffer == '':
-            return
-        if self.offset:
-            l = self.writeSomeData(lazyByteSlice(self.dataBuffer, self.offset),
-                                   self._ttl, self._inorder)
-        else:
-            l = self.writeSomeData(self.dataBuffer, self._ttl, self._inorder)
-
-        # There is no writeSomeData implementation in Twisted which returns
-        # < 0, but the documentation for writeSomeData used to claim negative
-        # integers meant connection lost.  Keep supporting this here,
-        # although it may be worth deprecating and removing at some point.
-        if isinstance(l, Exception) or l < 0:
-            return l
-        self.offset += l
-        # If there is nothing left to send,
-        if self.offset == len(self.dataBuffer) and not self._tempDataLen:
-            self.dataBuffer = b""
-            self.offset = 0
-            # stop writing.
-            self.stopWriting()
-            # If I've got a producer who is supposed to supply me with data,
-            if self.producer is not None and ((not self.streamingProducer)
-                                              or self.producerPaused):
-                # tell them to supply some more.
-                self.producerPaused = 0
-                self.producer.resumeProducing()
+                return main.CONNECTION_LOST
             elif self.disconnecting:
-                # But if I was previously asked to let the connection die, do
+                # But if I was previously asked to
+                #let the connection die, do
                 # so.
                 return self._postLoseConnection()
             elif self._writeDisconnecting:
                 # I was previously asked to half-close the connection.  We
                 # set _writeDisconnected before calling handler, in case the
-                # handler calls loseConnection(), which will want to check for
+                # handler calls loseConnection(), which
+                #will want to check for
                 # this attribute.
                 self._writeDisconnected = True
                 result = self._closeWriteConnection()
                 return result
+
         return None
+        
+
+    def doWrite(self):
+        """
+        Called when data can be written. It will send all messages recorded
+        in the buffer in a non-blocking way.
+        @return: C{None} on success, an exception or a negative integer on
+            failure.
+
+        @see: L{twisted.internet.interfaces.IWriteDescriptor.doWrite}.
+        """
+
+        # Send as much data as you can.
+        if len(self.messageBuffer) == 0:
+            return None
+        result = self.splitAndSend()
+        if len(self.messageBuffer) == 0:
+            #self.stopWriting()
+            #Really stop writable event
+            self.reactor._poller.remove_usock(self.socket.UDTSOCKET)
+            self.reactor._poller.add_usock(self.socket.UDTSOCKET,
+                                           self.reactor._POLL_IN)
+        return result
 
 
     def _closeWriteConnection(self):
@@ -783,8 +767,20 @@ class Client(Connection, tcp._BaseBaseClient):
         status = self.socket.getsockopt(udt.UDT_STATE)
         if (status != udt.UDTSTATUS_CONNECTED) and \
                 (status != udt.UDTSTATUS_INIT):
-            self.failIfNotConnected(error.getConnectError((
-                        staus)))
+            # self.failIfNotConnected(error.getConnectError((
+            #             status)))
+            #FIXME works for now
+            self.stopConnecting()
+            return
+        elif status == udt.UDTSTATUS_CONNECTED:
+            #Check for errors
+            #We're connected, finish the connection
+            del self.doWrite
+            del self.doRead
+            # we first stop and then start, to reset
+            #any references to the old doRead
+            self.startReading()
+            self._connectDone()
             return
 
         # doConnect gets called twice.  The first time we actually need to
@@ -795,45 +791,40 @@ class Client(Connection, tcp._BaseBaseClient):
         # is not /particularly/ detrimental to do so.  This should get
         # cleaned up some day, though.
         try:
-            connectResult = self.socket.connect_ex(self.realAddress)
+            connectResult = self.socket.connect(self.realAddress)
         except udt.UDTException as ue:
             connectResult = ue[0]
+
+        #TODO
         if connectResult:
-            if connectResult == udt.EISCONN:
-                pass
-            elif connectResult in (EASYNCRCV,):
+            if connectResult in (EASYNCRCV,):
+                #We're starting to connect to each other
                 self.startReading()
-                self.startWriting()
+                # self.startWriting()
                 return
             else:
                 self.failIfNotConnected(error.getConnectError((
                             connectResult, connectResult)))
                 return
-
-
-
-        status = self.socket.getsockopt(udt.UDT_STATE)
-        if status == udt.UDTSTATUS_CONNECTED:
-            # If I have reached this point without raising
-            #or returning, that means
-            # that the socket is connected.
-            del self.doWrite
-            del self.doRead
-            # we first stop and then start, to reset
-            #any references to the old doRead
-            self.startReading()
-            self.startWriting()
-            self._connectDone()
         else:
-            # If I have reached this point without raising
-            #or returning, that means
-            # that the socket is connected.
-            del self.doWrite
-            del self.doRead
-            # we first stop and then start, to reset
-            #any references to the old doRead
-            #self.startReading()
-            self.startWriting()
+            #FIXME but it works for now
+            opt = self.socket.getsockopt(udt.UDT_STATE)
+            if opt == udt.UDTSTATUS_CONNECTING:
+                #add monitoring to the reactor
+                self.startReading()
+                self.startWriting()
+                self.reactor._poller.add_usock(self.socket.UDTSOCKET,
+                                               self.reactor._POLL_OUT)
+            elif opt == udt.UDTSTATUS_CONNECTED:
+                #Check for errors
+                #We're connected, finish the connection
+                del self.doWrite
+                del self.doRead
+                # we first stop and then start, to reset
+                #any references to the old doRead
+                self.startReading()
+                self._connectDone()
+                return
 
     def _connectDone(self):
         """
@@ -863,7 +854,7 @@ class Client(Connection, tcp._BaseBaseClient):
                 self.stopReading()
                 self.stopWriting()
                 self.startReading()
-                self.startWriting()
+                #self.startWriting()
                 return
             Connection.doWrite(self)
 
